/*
 * wmdimp
 * Copyright (c) 2004 Daniele Tamino <aslo@wmdimp.cjb.net>.
 * This software covered by the GPL.  See COPYING file for details.
 *
 * debug.h
 *
 * This file contains all the debugging facilities
 *
 */
#ifndef debugH
#define debugH

#ifndef NDEBUG
#  define DBG(x) x
#  define IFDBG(dbg, release) dbg
#else
#  define DBG(x)
#  define IFDBG(dbg, release) release
#endif

#ifndef NDEBUG
#include <sys/time.h>

static inline double __now(void) {
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return tv.tv_sec + 0.000001 * tv.tv_usec;
}
static inline double __diff_from_now(struct timeval* tv)
{
    double time = tv->tv_sec + tv->tv_usec * 0.000001;
    return time - __now();
}

#ifndef DBG_MESSAGES
#  define DBG_MESSAGES 1
#endif

/* initialize debugging infrastructure */
void dbg_init(int argc, char** argv);

#if DBG_MESSAGES
int dbg_printf(const char* file, int line, const char* fmt, ...);

#  define DBGMSG(x...) dbg_printf(__FILE__, __LINE__, x)
#else
#  define DBGMSG(x...)
#endif

#ifndef DBG_MEMORY
#  define DBG_MEMORY 1
#endif

#if DBG_MEMORY

#include <mcheck.h>
/* mtrace reports that strdup allocated something that's not freed, but we
   want to know from where it was called ... */

#define strdup(s) \
  (__extension__                         \
    ({                                           \
        const char* __str = s; /* 1-time eval */ \
    int __idx, __len = strlen(__str);        \
        char *__result = malloc(__len + 1);      \
        for(__idx = 0; __idx <= __len; ++__idx)  \
        __result[__idx] = __str[__idx];      \
    __result;                                \
    }))

#endif

#ifndef DBG_METER
#  define DBG_METER 0
#endif

#if DBG_METER
/* show a debugging window with a percent meter */
void* dbg_meter_new(const char* name);
void dbg_meter_show(void* id, int percent);
void dbg_meter_delete(void* id);
#else
static inline void* dbg_meter_new(const char* name)
{ (void) name; return NULL; }
static inline void dbg_meter_show(void* id, int percent)
{ (void) id; (void) percent; }
static inline void dbg_meter_delete(void* id)
{ (void) id; }
#endif

/* debug this program in case of an inconsistency found (and where a
   core file wouldn't be enough */
void dbg_start_debugger(void);

#else /* NDEBUG */

#  define DBGMSG(x...)

#endif /* NDEBUG */

#endif
