#!/usr/bin/perl
$manpage="wmdimprc.1";
$infile="sample.wmdimprc";
$docstr="";
$examp="";
$option="";

open (FILE, $infile) or die("Error opening $infile for reading.\n");
open (OF, "> $manpage") or die("Error opening $manpage for writing.\n");

printf OF <<EOF
.\\" wmdimprc man page, configuration for wmdimp
.TH wmdimprc 1 "2003-12-27" "WMDIMP" "DIgital Music Player"
.SH NAME
~/.wmdimprc - configuration file for
.BR wmdimp (1).
.SH SYNOPSYS
.P
Each line is of the form
.TP
name = value;
.P
where
.I name
is the name of a configuration option (variable), and
.I value
its value (you'd guessed it? :\-).
.P
if 
.I name
is a variable which accepts multiple values, separate them
with commas (\`,') like in:
.TP
name = val1, val2, val3;
.P
You must escape or quote characters + : [ ] and spaces if a string
contains them (they are used internally).
Quoting can be done with single ' or double " quote,
escaping with \`\\\\'.
.br
Examples 'single\ quoted', "double\ quoted",
\\\\[escaped\\\\\\ characters\\\\].
.P
This page is automatically generated from
.BR sample.wmdimprc ,
found in wmdimp package. See that one for more information.

.SH OPTIONS

.P
Here are all the supported options in alphabetical order:

EOF
;

sub check_and_add_option {
    if($option) {
        $str = ".TP\n.B $option\n$docstr\n";
        $str .= ".B Examples:\n.br\n$examp\n" if $examp;
        $opttab{$option} = $str;
        $option="";
        $docstr="";
        $examp="";
    }
}

$trim=0;
sub trim($) {
    my $str = \@_[0];
    #print "untrimmed: '$$str'\n";
    $$str =~ s/^\s*//; 
    $$str =~ s/\s*$//; 
    #print "trimmed: '$$str'\n";
}
while(<FILE>) {
    next if /^\#\#/; # double `#' is ignored
    if(/^(\#?)([a-zA-Z0-9_]*) *=/) {
        # line like '[#] option = [value]'
	$option = $2;
        if($1) {
            /^\#(.*)$/;
            $examp .= $1."\n.br\n";
        }
    } elsif(/^\#(.*)$/) {
	check_and_add_option;
        $str = $1;
        # remove leading and trailing space if needed
        trim($str) if($trim);
        # always remove the first space character, if any
        $str =~ s/^ //;
	if($str || $docstr) {
	    # check for `\' at end of (trimmed) line
	    $cont = ($str =~ /^(.*)\\$/);
	    $str = $1 if($cont);
	    $docstr .= $str."\n";
	    # if newline isn't escaped, put a .br
	    if(!$cont) { $docstr .= ".br\n"; $trim=0; }
	    else { $trim=1; }
	}
    } else {
	check_and_add_option;
	$docstr = "";
    }
}

close(FILE);

@optlist = sort(keys opttab);
foreach $opt (@optlist) {
    $str=$opttab{$opt};
    print OF $str;
}

print OF ".SH SEE ALSO\n.BR wmdimp (1)\n";
close(OF);
