/*
 * wmdimp
 * Copyright (c) 2004 Daniele Tamino <aslo@wmdimp.cjb.net>
 *
 * Based on wmmp3: Copyright (c)1999 Patrick Crosby <xb@dotfiles.com>.
 *
 * This software covered by the GPL.  See COPYING file for details.
 *
 * mpg123_player.c
 *
 * player that uses "mpg123 -R dummy" to play mp3's
 *
 */
#ifndef mpg123_playerH
#define mpg123_playerH

#include "player.h"
#include <stdio.h>

#ifndef NDEBUG
#define ERROR_CHECKING_MTX
#endif

typedef struct mpg123_player {
    player pl; /* incorporate the player struct */
    /* pl.status is the status of the player as seen externally */
    int curr_status; /* actual status of mpg123 */
    int wanted_status; /* status we'd like to be in soon */

    volatile int error;
    volatile int play_pid;

    FILE* play_rd; /* pipe to read from */
    FILE* play_wr; /* pipe to write to */

    pthread_t play_thr;   /* thread to read mpg123 output */
    int sync; /* playing synchronously (play_thr == main thread) */

    char* filename; /* current filename */
    /* song info: if valid, always apply to the current filename */
    char* volatile song_title; /* play_thread puts here the song title */
    volatile struct song_pos song_len;

    volatile struct song_pos curr_pos;

    player_cb_func callback;
    void* cb_param;
    /* mutex to avoid being interrupted by play_thread  */
    pthread_mutex_t nointr_mtx;
    /* mutex to access song title (main thread and play_thread) */
    pthread_mutex_t title_mtx;
    /* mutex to access curr_pos (main thread and play_thread) */
    pthread_mutex_t curr_pos_mtx;

    /* generic condition used to synchronize play_thread and main thread */
    pthread_mutex_t play_mtx; /* with associated mutex */
    pthread_cond_t play_cond;
    
} mpg123_player;

extern player_factory mpg123_player_factory;

#endif
