/*
 * wmdimp
 * Copyright (c) 2004 Daniele Tamino <aslo@wmdimp.cjb.net>.
 * This software covered by the GPL.  See COPYING file for details.
 *
 * player_mad.c
 *
 * mp3 file player using libmad and output.c to play sound (header file).
 *
 */
#ifndef mad_playerH
#define mad_playerH

#include "player.h"
#include "thrcomm.h"
#include <mad.h>

enum mad_msg_tx_value {
    MSG_NONE,
    MSG_TERM, /* terminate decoder thread */
    MSG_STOP, /* stop playing and wait for next song (used?) */
    MSG_PAUSE,
    MSG_RESUME, /* resume if paused, else nothing */
    MSG_SONGCHANGE, /* input file has changed, do what's to do */
    MSG_SEEK /* look at dec_seek to get new position */
};

enum mad_msg_rx_value {
   ACK_NONE,
   ACK_PAUSE, /* in response to MSG_PAUSE */
   ACK_ERROR, /* an error occurred (used handling SEEK,SONGCHANGE) */
   ACK_DONE /* decoder handled the request */   
};

typedef struct player_mad
{
    player pl;

    /* song info  */
    song_pos pos, len; /* position and length */
    long bitrate; /* bitrate (in bits per second) */
    pthread_mutex_t pos_mtx; /* locks position */
    /* no need to lock len and bitrate since these are modified only by
       scan_thread: just pthread_join it */

    player_cb_func callback;
    void* cb_param;

    /* input file data */
    char* filename; /* input file name (needed for scan_thread) */
    char* title; /* title (get from ID3 or something) */
    pthread_mutex_t title_mtx;
    int infd; /* input file descriptor  */
    time_t mtime; /* last modif. time when opened */
    long flen;
    int eof;
    int mmap; /* == 1 if we used mmap */
#ifdef HAVE_MMAP
    void* buffer; /* mmap'ed file address */
#endif
    pthread_mutex_t infile_mtx;

    pthread_t scan_thr;
    volatile int scan_thr_running; /* ==1 while scan_thr is running */
    volatile int abort_scan; /* signals scan_thr to abort */
    pthread_t dec_thr;
    thrchannel dec_thr_chn; /* channel to communicate with dec_thr */
    pthread_mutex_t my_mtx; /* mutex on changing myself */
} player_mad;

extern player_factory player_mad_factory;

#endif
