/*
 * wmdimp
 * Copyright (c) 2004 Daniele Tamino <aslo@wmdimp.cjb.net>
 *
 * This software covered by the GPL.  See COPYING file for details.
 *
 * timer.h
 *
 * function to asynchronously run a handler at a given time (header file)
 *
 */
#ifndef timerH
#define timerH

/* N.B. You _can_ call timer_schedule/stop/change* within timer handler */

/* N.B. on linux-i386, timer is accurate only to 0.01 seconds */

/* tv is the time the handler requested to be ran (as near as possible
   to current time), param is an arbirtary parameter from timer_schedule */
typedef void (*timer_handler)(struct timeval* tv, void* param);

/* schedule a job for execution at time tv, return job id, or -1 on error */
int timer_schedule(struct timeval* tv, timer_handler th, void* param);

/* change parameters of a job (return 0 on success, -1 on error) */
/* N.B. error means invalid job id, often this is a bug */
int timer_changetime(int jobid, struct timeval* tv); /* N.B. O(log n) */
int timer_changehndl(int jobid, timer_handler th); /* O(1) */
int timer_changeparam(int jobid, void* param); /* O(1) */

/* cancel a timer job */
int timer_cancel(int jobid); /* O(log n) */

/* start the timer thread */
int timer_start(void);

/* stop the timer thread */
int timer_stop(void);

#endif
